import ifcfg


def run():
    from PyInquirer import prompt
    import os
    questions = [
        {
            'type': 'list',
            'name': 'user_option',
            'message': 'Network tools (Use arrow keys)',
            'choices': ["View network information", "difference", "product", "divide"]
        },
    ]
    answers = prompt(questions)
    os.system('clear')
    if answers.get("user_option") == "View network information":
        questions = [
            {
                'type': 'list',
                'name': 'user_option',
                'message': 'Which info would you like to view (Use arrow keys)',
                'choices': ["IP", "Router", "Subnet-mask"]
            },
        ]
        answers = answers = prompt(questions)
        os.system('clear')
        if answers.get("user_option") == "IP":
            device_list = []
            for name, interface in ifcfg.interfaces().items():
                device_list.append(interface['device'])
            if len(device_list) > 0:
                questions = [
                    {
                        'type': 'list',
                        'name': 'user_option',
                        'message': 'Choose an interface (Use arrow keys)',
                        'choices': ["default interface"] + device_list,
                        'default': 'default'
                    },
                ]
                answers = answers = prompt(questions)
                default = ifcfg.default_interface()
                if answers.get("user_option") == "default":
                    print("-", default['device'])  # Device name
                    print("  - IPv4")  # First IPv4 found
                    if len(default['inet4']) > 0:
                        for i in default['inet4']:
                            print("     -", i)
                    else:
                        print("     - None.")
                    print("  - IPv6")  # First IPv4 found
                    if len(default['inet6']) > 0:
                        for i in default['inet6']:
                            print("     -", i)
                    else:
                        print("     - None.")
                else:
                    for name, interface in ifcfg.interfaces().items():
                        if interface['device'] == answers.get("user_option"):
                            print("-", interface['device'])  # Device name
                            print("  - IPv4")  # First IPv4 found
                            if len(interface['inet4']) > 0:
                                for i in interface['inet4']:
                                    print("     -", i)
                            else:
                                print("     - None.")
                            print("  - IPv6")  # First IPv4 found
                            if len(interface['inet6']) > 0:
                                for i in interface['inet6']:
                                    print("     -", i)
                            else:
                                print("     - None.")
                        # print(interface['netmask'])  # Backwards compat: First netmask
                        # print(interface['netmasks'])  # List of netmasks
                        # print(interface['broadcast'])  # Backwards compat: First broadcast
                        # print(interface['broadcasts'])  # List of broadcast
                        # default = ifcfg.default_interface()
            else:
                print("No device detected!")

def get_network_info():
    import ifcfg
    for i in (ifcfg.interfaces()):
        print(i)



import argparse
from hallon import lang


def show_welcome(l):
    print(lang.get_msg("welcome_msg", l))


def show_version(l):
    print(lang.get_msg("version_msg", l))


def generate_topology(l, args):
    from hallon import topo
    topo.run(l, args)


def run():
    parser = argparse.ArgumentParser()
    parser.add_argument("-l", "--cli_lang", type=str, help="client language", default=lang.get_default_lang())
    parser.add_argument("-v", "--version", help="client version", action='store_true')
    parser.add_argument("-t", "--thread_num", type=int, help="number of threads", default=1)
    parser.add_argument("-nt", "--network_topology", help="generate topology", action='store_true')
    parser.add_argument("-cadr", "--core_address", type=str, help="telnet address of the core switch")
    parser.add_argument("-cprt", "--core_port", type=int, help="telnet port of the core switch", default=23)
    parser.add_argument("-cusr", "--core_username", type=str, help="username for logging in to the core switch", default="admin")
    parser.add_argument("-cpas", "--core_password", type=str, help="password for logging in to the core switch")
    parser.add_argument("-cenpas", "--core_enable_password", type=str, help="enable password for the core switch")
    parser.add_argument("-uprt", "--unified_port", type=int, help="general telnet port", default=23)
    parser.add_argument("-uusr", "--unified_username", type=str, help="general username for telnet login")
    parser.add_argument("-upas", "--unified_password", type=str, help="general password for telnet login")
    parser.add_argument("-uenpas", "--unified_enable_password", type=str, help="general enable password for telnet")

    args = parser.parse_args()

    cli_lang = args.cli_lang.strip().lower()
    multi_proc_num = args.thread_num
    version_display = args.version
    network_topology = args.network_topology

    if version_display:
        show_version(cli_lang)
        exit()

    if network_topology:
        if args.network_topology and (args.core_address is None or args.core_password is None or args.core_enable_password is None):
            parser.error("--network_topology requires --core_address, --core_password and --core_enable_password")
        else:
            if args.unified_username is None:
                args.unified_username = args.core_username
            if args.unified_password is None:
                args.unified_password = args.core_password
            if args.unified_enable_password is None:
                args.unified_enable_password = args.core_enable_password
            from hallon import func
            if func.is_ipv4(args.core_address.strip()) and 0 <= args.core_port <= 65536:
                generate_topology(cli_lang, args)
            else:
                parser.error("--core_address/--core_port invalid.")
        exit()

    if cli_lang not in lang.get_lang():
        # raise ValueError("Bad argument: -l, supported language:", lang.get_lang())
        print("Bad argument: -l, supported languages:", lang.get_lang(), ", default language:", lang.get_default_lang())
        exit()



import telnetlib


def core_login(ip, user, pas, timeout=5, port=23):
    # print("Telnet", ip, "...", end="")
    try:
        tn = telnetlib.Telnet(ip, port=port, timeout=timeout)
        # tn.set_debuglevel(1)
    except:
        # print("Connection timeout.")
        return 0
    try:
        tn.read_until(b'Username:')
        tn.write(user.encode('utf-8') + b'\n')
        tn.read_until(b'Password:')
        tn.write(pas.encode('utf-8') + b'\n')
        info = tn.read_until(b'>', timeout=5)
        try:
            info = info.decode('gb2312')
        except:
            try:
                info = info.decode('utf-8')
            except:
                tn.close()
                return -2
        if ">" not in info:
            # print("Invalid credentials.")
            tn.close()
            return -1
        host_name = info[info.index("\n"):].strip()[:-1]
        # print("Login successful, device name:", host_name)
        return [tn, host_name]
    except:
        # print("Failed.")
        tn.close()
        return -1


def tel_login(tn, ip, user, pas, timeout=1, port=23):
    print("Telnet", ip, "...", end="")
    command = "telnet " + ip + " " + str(port)
    try:
        tn.write(command.encode('utf-8') + b'\n')
        info = tn.read_until(b'Username:', timeout=5)
        try:
            info = info.decode('gb2312')
        except:
            try:
                info = info.decode('utf-8')
            except:
                print("Decoding error.")
                tn.close()
                return -2
        if "Username" not in info:
            print("Connection timeout.")
            tn.close()
            return 0
        tn.write(user.encode('utf-8') + b'\n')
        tn.read_until(b'Password:')
        tn.write(pas.encode('utf-8') + b'\n')
        info = tn.read_until(b'>', timeout=5)
        try:
            info = info.decode('gb2312')
        except:
            try:
                info = info.decode('utf-8')
            except:
                print("Decoding error.")
                tn.close()
                return -2
        if ">" not in info:
            print("Invalid credentials.")
            tn.close()
            return -1
        host_name = info[info.index("\n"):].strip()[:-1]
        print("Login successful, device name:", host_name)
        return [tn, host_name]
    except:
        print("Failed.")
        tn.close()
        return -1


def enable(tn, host_name, enpas):
    # print("Enabling", host_name, "...", end="")
    try:
        tn.write("enable".encode('utf-8') + b'\n')
        tn.read_until(b'Password:')
        tn.write(enpas.encode('utf-8') + b'\n')
        tn.read_until(bytes(host_name + '#', 'utf8'))
        # print("Enabled.")
        return tn
    except:
        # print("Enter enable mode failed.")
        tn.close()
        return 0


def lldp_lookup(tn, host_name):
    try:
        # print("Looking for neighbors of", host_name + "...", end="")
        tn.write("terminal length 0".encode('utf-8') + b'\n')
        tn.read_until(bytes(host_name + '#', 'utf8'))
        tn.write("show lldp neighbors detail".encode('utf-8') + b'\n')
        info = tn.read_until(bytes(host_name + '#', 'utf8'))
        try:
            info = info.decode('gb2312')
        except:
            try:
                info = info.decode('utf-8')
            except:
                # print("Decoding error.")
                tn.close()
                return -1
        # print(info.strip().strip("show lldp neighbors detail").strip(host_name+"#").strip().split("----------------------------------------------------------------------------"))
        lldp_list_raw = info.strip().strip("show lldp neighbors detail").strip(host_name + "#").strip().split(
            "----------------------------------------------------------------------------")
        lldp_list = []
        for i in lldp_list_raw:
            if i.strip() != "":
                lldp_list.append(i.strip())
        if len(lldp_list) > 1:
            lldp_num = int(len(lldp_list) / 2)
            for index, i in enumerate(lldp_list):
                if ((index + 1) % 2) != 0:
                    lldp_list[index] = i[i.index("[") + 1:i.index("]")].strip()
                else:
                    lldp_list_detail = []
                    for n in i.split("\n"):
                        if n.strip() != "":
                            if ":" in n.strip():
                                lldp_list_detail_raw = []
                                for m in n.strip().split(":"):
                                    lldp_list_detail_raw.append(m.strip())
                                lldp_list_detail.append(lldp_list_detail_raw)
                    lldp_list[index] = lldp_list_detail
        else:
            lldp_num = 0
        lldp_dict = {"lldp_num": lldp_num,
                     "lldp_detail": {}
                     }
        for index, i in enumerate(lldp_list):
            lldp_dict_single = {}
            if (index + 1) % 2 != 0:
                lldp_dict_single["port"] = i
                lldp_dict_pro = {}
                for n in lldp_list[index + 1]:
                    lldp_dict_pro[n[0]] = n[1]
                lldp_dict_single["info"] = lldp_dict_pro
                lldp_dict["lldp_detail"][str(int((index + 2) / 2))] = lldp_dict_single
        # print(lldp_dict["lldp_num"], "neighbors found.")
        return [tn, lldp_dict]
    except:
        # print("Failed.")
        tn.close()
        return 0


def show_interface_status(tn, host_name):
    try:
        tn.write("terminal length 0".encode('utf-8') + b'\n')
        tn.read_until(bytes(host_name + '#', 'utf8'))
        tn.write("show interface".encode('utf-8') + b'\n')
        info = tn.read_until(bytes(host_name + '#', 'utf8'))
        try:
            info = info.decode('gb2312')
        except:
            try:
                info = info.decode('utf-8')
            except:
                tn.close()
                return -1
        int_list_raw = info.strip().strip().strip("show interface").strip(host_name + "#").split("\n=")
        int_list_raw_pro = []
        # print(len(int_list_raw))
        for i in int_list_raw:
            if i.strip() != "":
                int_list_raw_pro.append(i.strip())
        for index, i in enumerate(int_list_raw_pro):
            int_list_raw_pro[index] = i.split("\n", 1)
        if len(int_list_raw_pro) > 1:
            for index, i in enumerate(int_list_raw_pro):
                int_list_raw_pro[index][0] = i[0].replace("=", "").strip()
                int_list_raw_pro[index][1] = i[1][:i[1].index("Rxload")]
        for i in int_list_raw_pro:
            print(i[0])
            print(i[1])
    except:
        tn.close()
        return 0


def disconnect(tn):
    tn.close()



from setuptools import setup


def readfile(filename):
    with open(filename, 'r+') as f:
        return f.read()


setup(
    name="hallon",
    version="0.0.1",
    description="hallon is a package for network debugging",
    long_description=readfile('README.md'),
    long_description_content_type='text/markdown',
    author="Ryan Alloriadonis",
    author_email="yuantai78@gmail.com",
    url="https://github.com/RyanYuanSun/hallon",
    py_modules=['hallon'],
    python_requires='>=3.7',
    packages=['hallon'],
    license=readfile('LICENSE'),
    install_requires=[
      ],
    entry_points={
        'console_scripts': ['hallon=hallon:cmd', 'hallon-tools=hallon:cmd_tool']
        },
)


lang_list = ['en', 'zh']
default_lang = "en"
scripts = {"welcome_msg": ["Hallon Networks Command Line Tools", "汉龙网络命令行工具"],
           "version_msg": ["0.0.1"]
           }


def get_lang_index(l):
    return lang_list.index(l)


def get_lang():
    return lang_list


def get_default_lang():
    return default_lang


def get_msg(msg, l):
    if l in get_lang():
        return scripts[msg][get_lang_index(l)]
    else:
        return get_msg(msg, get_default_lang())




from hallon import exception
from hallon import switch


def run(l, args):
    from hallon import cli
    cli.show_welcome(l)
    main(args)


def get_lldp_dict(target):
    print("123")


def login_to_core_and_enable(core_ip, core_user, core_pass, timeout, core_port, core_en_pass):
    # login to core switch
    tn_ini = switch.core_login(core_ip, core_user, core_pass, 1, core_port)
    if not tn_ini:
        return 0
    else:
        host_name = tn_ini[1]
        tn = tn_ini[0]

        # enter enable mode
        tn = switch.enable(tn, host_name, core_en_pass)
        if not tn:
            return 0
        else:
            return [tn, host_name]


def main(args):
    from hallon import switch

    lldp_all = {}
    lldp_search_tree = []
    lldp_search_tree_history = []

    # parse arguments
    core_ip = args.core_address
    core_pass = args.core_password.strip()
    core_port = args.core_port
    core_en_pass = args.core_enable_password.strip()
    core_user = args.core_username.strip()
    uni_user = args.unified_username.strip()
    uni_pass = args.unified_password.strip()
    uni_port = args.unified_port
    uni_en_pass = args.unified_enable_password.strip()

    lldp_search_tree.append(core_ip)
    lldp_search_tree_history.append(core_ip)
    tn_obj = login_to_core_and_enable(core_ip, core_user, core_pass, 1, core_port, core_en_pass)
    if not tn_obj:
        exit()
    host_name = tn_obj[1]
    tn = tn_obj[0]

    # get lldp info of core
    tn_lldp_obj = switch.lldp_lookup(tn, host_name)
    if not tn_lldp_obj:
        exit()
    tn = tn_lldp_obj[0]
    switch.disconnect(tn)
    lldp_dict = tn_lldp_obj[1]
    lldp_all[core_ip] = lldp_dict
    if lldp_dict["lldp_num"] > 0:
        for i in lldp_dict['lldp_detail']:
            if "Management address" in lldp_dict["lldp_detail"][i]["info"].keys():
                if lldp_dict["lldp_detail"][i]["info"]["Management address"] not in lldp_search_tree_history:
                    lldp_search_tree.append(lldp_dict["lldp_detail"][i]["info"]["Management address"])
    lldp_search_tree.remove(core_ip)

    # start depth-first-search
    while len(lldp_search_tree) > 0:
        for ip in lldp_search_tree:
            print("Pending:", len(lldp_search_tree))
            print("Archived:", len(lldp_all))
            print("Failed:", len(lldp_search_tree_history) - len(lldp_all))
            print("Total processed:", len(lldp_search_tree_history))
            print(lldp_search_tree)
            print()
            lldp_search_tree_history.append(ip)
            tn_obj = login_to_core_and_enable(core_ip, core_user, core_pass, 1, core_port, core_en_pass)
            if not tn_obj:
                pass
            else:
                tn = tn_obj[0]
                host_name = tn_obj[1]

                # making telnet connection
                tn_ini = switch.tel_login(tn, ip, uni_user, uni_pass, 1, uni_port)
                if not tn_ini:
                    pass
                else:
                    tn = tn_ini[0]
                    host_name = tn_ini[1]
                    tn = switch.enable(tn, host_name, uni_en_pass)
                    if not tn:
                        pass
                    else:
                        # get lldp info of core
                        tn_lldp_obj = switch.lldp_lookup(tn, host_name)
                        if not tn_lldp_obj:
                            pass
                        else:
                            tn = tn_lldp_obj[0]
                            switch.disconnect(tn)
                            lldp_dict = tn_lldp_obj[1]
                            lldp_all[ip] = lldp_dict
                            if lldp_dict["lldp_num"] > 0:
                                for i in lldp_dict['lldp_detail']:
                                    if "Management address" in lldp_dict["lldp_detail"][i]["info"].keys():
                                        if lldp_dict["lldp_detail"][i]["info"]["Management address"] not in lldp_search_tree_history and lldp_dict["lldp_detail"][i]["info"]["Management address"] not in lldp_search_tree:
                                            lldp_search_tree.append(lldp_dict["lldp_detail"][i]["info"]["Management address"])
            lldp_search_tree.remove(ip)

    # get interface info
    # tn = switch.show_interface_status(tn, host_name)
